<?php

# File: CL_expand.php Updated April 6th, 2017
# Author: Eoin Fahy
#
# Copyright (C) 2006-2014. The Regents of the University of California. All Rights Reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the LIPID MAPS (TM) consortium nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The LIPID MAPS (TM) consortium is a multi-institutional multi-year effort
# involved in identification and characterization of existing and novel lipids,
# quantifications of changes in their metabolites, and development of biochemical
# pathways and interaction network maps involving lipids. Please acknowledge use
# of the LIPID MAPS (TM) Tools in your work by citing appropriate literature
# references. For further details, please visit www.lipidmaps.org.
#
# Support for this effort is provided by  by National Institutes of Health (NIH)
# and National Institute of General Medical Sciences (NIGMS) Glue Grant
# NIH/NIGMS Grant 1 U54 GM69338.
#
#USAGE: php CL_expand.php <none|chains> <ion type>
#USAGE: Permitted ion type : M.,M+H,M+2H,M+3H,M+4H,M-H,M-2H,M-3H,M-4H,M+NH4,M+Ag,M+Na,M+2Na,M+K,M+2K,M+Li,M+2Li,M.Cl,M.OAc,M-DFPU+NH4,M-diDFPU+NH4
#USAGE: ion types correspond to: [M.]+,[M+H]+,[M+2H]2+,[M+3H]3+,[M+4H]4+,[M-H]-,[M-2H]2-,[M-3H]3-,[M-4H]4-,[M+NH4]+,[M+Ag]+,[M+Na]+,[M+2Na]2+,[M+K]+,[M+2K]2+,[M+Li]+,[M+2Li]2+,[M+Cl]-,[M+OAc]-,[M-DFPU+NH4]+,[M-diDFPU+NH4]+
#Example: php CL_expand.php > CL.txt (neutral molecule mass calculation,individual chains are not specified)
#Example: php CL_expand.php chains > CL.txt (neutral molecule mass calculation,individual chains are specified)
#Example: php CL_expand.php none M+H > CL.txt ([M+H]+ ion mass calculation,individual chains are not specified)
#Example Save to file: php CL_expand.php > CL.txt

$chains = isset($argv[1]) ? $argv[1] : '';
$ion = isset($argv[2]) ? $argv[2] : '';

$sub_exactmass= 0; 
$theion='';
$mult_factor=1;
$num_00=0;

include("ioncalc.php");

##############################SPECIFY CHAINS###################################
#Add or subtract chains to this list as required (one per line)
$sn1list=
'14:0
14:1
16:0
16:1
18:0
18:1
18:1
18:2
18:3
18:4
20:0
20:1
20:2
20:3
20:4
20:5
22:0
22:1
22:2
22:4
22:5
22:6
24:0
O-16:0
P-16:0';

#########################################################################

$sn2list=preg_replace("/[OP]-[0-9\(\):EZ]+/","",$sn1list); #dont allow ethers at sn2 or sn3

$CL2_list=explode("\n",$sn2list);
$CL3_list =$CL2_list;
$CL4_list =$CL2_list;

if(preg_match("/^0:0\n/",$sn1list) || preg_match("/\n0:0\n/",$sn1list)){
$num_00=1;
}

$CL1_list=explode("\n",$sn1list);
$num_chains =count($CL1_list) +$num_00;
file_put_contents('php://stderr', "Number of chains: $num_chains\n");

$uniq=array();
$cnt=0;

for($i = 0; $i < count($CL1_list); $i++) {
#file_put_contents('php://stderr', "$CL1_list[$i]\n");
	for($j = 0; $j < count($CL2_list); $j++) {
		for($k = 0; $k < count($CL3_list); $k++) {
			for($l = 0; $l < count($CL4_list); $l++) {
			
$CL_abbrev="$CL1_list[$i]". "/" . "$CL2_list[$j]" . "/" . "$CL3_list[$k]" . "/" . "$CL4_list[$l]";
$sn1=preg_replace("/\(.*/","",$CL1_list[$i]);
$sn2=preg_replace("/\(.*/","",$CL2_list[$j]);
$sn3=preg_replace("/\(.*/","",$CL3_list[$k]);
$sn4=preg_replace("/\(.*/","",$CL4_list[$l]);

$numC = 0;
$numH = 0;
$numO = 0;
$numN = 0;
$numP = 0;
$numNa = 0;
$numK = 0;
$numCl = 0;
$numAg = 0;
$numF = 0;
$numS = 0;
$numLi = 0;

$numacylC = 0;
$numacylH = 0;
$numacylO = 0;
$numDB = 0;
     $sn1mass=0;
     $sn2mass=0;
     $sn3mass=0;

                    
$sp1Array = explode(":",$CL1_list[$i]);
$sp2Array = explode(":",$CL2_list[$j]);
$sp3Array = explode(":",$CL3_list[$k]);
$sp4Array = explode(":",$CL4_list[$l]);


                    if(preg_match("/^O-/",$sp1Array[0])){
                        $sp1Array[0] = preg_replace("/^O-/", "",$sp1Array[0]);
                        $numacylH += 2;
                        $numacylO -= 1;
                    }                
                    
                    if(preg_match("/^O-/",$sp2Array[0])){
                        $sp2Array[0] = preg_replace("/^O-/", "",$sp2Array[0]);
                        $numacylH += 2;
                        $numacylO -= 1;
                    }                
                                        
                    if(preg_match("/^O-/",$sp3Array[0])){
                        $sp3Array[0] = preg_replace("/^O-/", "",$sp3Array[0]);
                        $numacylH += 2;
                        $numacylO -= 1;
                    }                
                    if(preg_match("/^O-/",$sp4Array[0])){
                        $sp4Array[0] = preg_replace("/^O-/", "",$sp4Array[0]);
                        $numacylH += 2;
                        $numacylO -= 1;
                    }    
                      if(preg_match("/^P-/",$sp1Array[0])){
                          $sp1Array[0] = preg_replace("/^P-/", "",$sp1Array[0]);
                          $numacylO -= 1;
                      }                
                      
                      if(preg_match("/^P-/",$sp2Array[0])){
                          $sp2Array[0] = preg_replace("/^P-/", "",$sp2Array[0]);
                          $numacylO -= 1;
                      }                
                                          
                      if(preg_match("/^P-/",$sp3Array[0])){
                          $sp3Array[0] = preg_replace("/^P-/", "",$sp3Array[0]);
                          $numacylO -= 1;
                    } 
                      if(preg_match("/^P-/",$sp4Array[0])){
                          $sp4Array[0] = preg_replace("/^P-/", "",$sp4Array[0]);
                          $numacylO -= 1;
                    } 
                    
                    if($sp1Array[0] > 0){
                        $numacylC += $sp1Array[0];
                        $numacylO += 1;
                        $numacylH += (($sp1Array[0] - 1) * 2) - ($sp1Array[1] * 2);
                   }                  
                    if($sp2Array[0] > 0){
                        $numacylC += $sp2Array[0];
                        $numacylO += 1;
                        $numacylH += (($sp2Array[0] - 1) * 2) - ($sp2Array[1] * 2);
                   }                    
                    
                    if($sp3Array[0] > 0){
                        $numacylC += $sp3Array[0];
                        $numacylO += 1;
                        $numacylH += (($sp3Array[0] - 1) * 2) - ($sp3Array[1] * 2);
                   }  

                    if($sp4Array[0] > 0){
                        $numacylC += $sp4Array[0];
                        $numacylO += 1;
                        $numacylH += (($sp4Array[0] - 1) * 2) - ($sp4Array[1] * 2);
                   }  
                   
##from glycerol backbone  
                    $numP = 2;
                    $numC = $numacylC + 9;
                    $numH = $numacylH + 22;
                    $numO = $numacylO + 13;
                    $numDB = $sp1Array[1] + $sp2Array[1] + $sp3Array[1] + $sp4Array[1];

$CL_mass = $mult_factor * ($sub_exactmass + ($numC * 12) + ($numH * 1.007825) + ($numO * 15.994915) + ($numP * 30.973763));
$CL_mass = sprintf('%0.4f',$CL_mass);                   

include("ioncalc2.php");


$formula=preg_replace("/([A-Za-z])1$/","$1",$formula);
$formula=preg_replace("/([A-Za-z]+)1([A-Za-z]+)/","$1$2",$formula);


  $nominal_mass=floor($CL_mass);  
  if(preg_match("/([OP])-/",$CL_abbrev,$MATCHES)){
  $CHAINTYPE=$MATCHES[1];
  $PREFIX=$CHAINTYPE .'-';
  }
  else{
  $CHAINTYPE='A';
  $PREFIX='';
  }
if($sn1 && $sn2 && $sn3 && $sn4){

if($sn1 == '0:0' && $sn2 == '0:0' && $sn3 == '0:0' && $sn4 == '0:0'){
#skip this one (no chains)
}
elseif(preg_match("/^[OP]-.+\/[OP]-/",$CL_abbrev)){
#ignore double ethers
}
elseif($chains == 'chains'){
print "$nominal_mass\t$CL_mass\t$numacylC\t$numDB\tCL($CL_abbrev)\t$sn1\t$sn2\t$sn3\t$sn4\t$formula\t$CHAINTYPE\t$theion\n";
}
else{
$full_abbrev_bulk="CL($PREFIX$numacylC:$numDB)";
if(!in_array($full_abbrev_bulk,$uniq)){
array_push($uniq,$full_abbrev_bulk);
print "$nominal_mass\t$CL_mass\t$full_abbrev_bulk\t$numacylC\t$numDB\t$formula\t$CHAINTYPE\t$theion\n";
}
}
}           

                   
                   
                    
		}
	}
}

}


?>

