<?php

# File: GL_expand.php Updated April 6th, 2017
# Author: Eoin Fahy
#
# Copyright (C) 2006-2014. The Regents of the University of California. All Rights Reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the LIPID MAPS (TM) consortium nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The LIPID MAPS (TM) consortium is a multi-institutional multi-year effort
# involved in identification and characterization of existing and novel lipids,
# quantifications of changes in their metabolites, and development of biochemical
# pathways and interaction network maps involving lipids. Please acknowledge use
# of the LIPID MAPS (TM) Tools in your work by citing appropriate literature
# references. For further details, please visit www.lipidmaps.org.
#
# Support for this effort is provided by  by National Institutes of Health (NIH)
# and National Institute of General Medical Sciences (NIGMS) Glue Grant
# NIH/NIGMS Grant 1 U54 GM69338.
#
#USAGE: php GL_expand.php <none|chains> <ion type> <Minimum carbons in chain> <Maximum carbons in chain> <minimum ratio of carbons in chain to double bonds>
#USAGE: Permitted ion type : M.,M+H,M+2H,M+3H,M+4H,M-H,M-2H,M-3H,M-4H,M+NH4,M+Ag,M+Na,M+2Na,M+K,M+2K,M+Li,M+2Li,M.Cl,M.OAc
#USAGE: ion types correspond to: [M.]+,[M+H]+,[M+2H]2+,[M+3H]3+,[M+4H]4+,[M-H]-,[M-2H]2-,[M-3H]3-,[M-4H]4-,[M+NH4]+,[M+Ag]+,[M+Na]+,[M+2Na]2+,[M+K]+,[M+2K]2+,[M+Li]+,[M+2Li]2+,[M+Cl]-,[M+OAc]-
#Example: php GL_expand.php > GL.txt (neutral molecule mass calculation,individual chains are not specified)
#Example: php GL_expand.php chains > GL.txt (neutral molecule mass calculation,individual chains are specified)
#Example: php GL_expand.php none M+H > GL.txt ([M+H]+ ion mass calculation,individual chains are not specified)
#Example: php GL_expand.php none M-H 14 26 3 > GL.txt ([M-H]- ion mass calculation,individual chains are not specified, C14-C26 range for chains, minimum ratio of carbons:double bonds=3.5 )

$chains = isset($argv[1]) ? $argv[1] : '';
$ion = isset($argv[2]) ? $argv[2] : '';
$C_min = isset($argv[3]) ? $argv[3] : 10;
$C_max = isset($argv[4]) ? $argv[4] : 34;
$MINDBRATIO = isset($argv[5]) ? $argv[5] : 3.5;

$sub_exactmass= 0; 
$theion='';
$mult_factor=1;
$num_00=0;
$chaincount1=0;
$chaincount2=0;

include("ioncalc.php");
##############################SPECIFY CHAINS###################################
#Chains in $sn1list only at the sn1 position
$sn1list='0:0
O-16:0
O-18:0
O-20:0
P-16:0
P-18:0
P-20:0
';
#Allow for DG, MG species
$sn2list='0:0
';
###############################################################################
for($i = $C_min; $i <= $C_max; $i++) {
for($j = 0; ($j/$i) <= 1/$MINDBRATIO; $j++) {
$sn1list .="$i" .':' . "$j\n";
$sn2list .="$i" .':' . "$j\n";
}
}


#$sn2list=preg_replace("/[OP]-[0-9\(\):EZ]+/","",$sn1list); #dont allow ethers at sn2 or sn3

$GL2_list=explode("\n",$sn2list);

$GL3_list =$GL2_list;

if(preg_match("/^0:0\n/",$sn1list) || preg_match("/\n0:0\n/",$sn1list)){
$num_00=1;
}

$GL1_list=explode("\n",$sn1list);


$uniq=array();
$chaincount1=count($GL1_list);
$chaincount2=count($GL2_list);
#file_put_contents('php://stderr', "Chain lengths from $C_min to $C_max. Number of chains: $chaincount1\n");

for($i = 0; $i < $chaincount1-1; $i++) {
#file_put_contents('php://stderr', "$GL1_list[$i]\n");
for($j = 0; $j < $chaincount2-1; $j++) {
		for($k = 0; $k < $chaincount2-1; $k++) {

$GL_abbrev="$GL1_list[$i]". "/" . "$GL2_list[$j]" . "/" . "$GL3_list[$k]";
$sn1=preg_replace("/\(.*/","",$GL1_list[$i]);
$sn2=preg_replace("/\(.*/","",$GL2_list[$j]);
$sn3=preg_replace("/\(.*/","",$GL3_list[$k]);

$numC = 0;
$numH = 0;
$numO = 0;
$numN = 0;
$numP = 0;
$numNa = 0;
$numK = 0;
$numCl = 0;
$numAg = 0;
$numF = 0;
$numS = 0;
$numLi = 0;

$numacylC = 0;
$numacylH = 0;
$numacylO = 0;
$numDB = 0;
     $sn1mass=0;
     $sn2mass=0;
     $sn3mass=0;

                    
$sp1Array = explode(":",$GL1_list[$i]);
$sp2Array = explode(":",$GL2_list[$j]);
$sp3Array = explode(":",$GL3_list[$k]);



                    if(preg_match("/^O-/",$sp1Array[0])){
                        $sp1Array[0] = preg_replace("/^O-/", "",$sp1Array[0]);
                        $numacylH += 2;
                        $numacylO -= 1;
                    }                
                    
                    if(preg_match("/^O-/",$sp2Array[0])){
                        $sp2Array[0] = preg_replace("/^O-/", "",$sp2Array[0]);
                        $numacylH += 2;
                        $numacylO -= 1;
                    }                
                                        
                    if(preg_match("/^O-/",$sp3Array[0])){
                        $sp3Array[0] = preg_replace("/^O-/", "",$sp3Array[0]);
                        $numacylH += 2;
                        $numacylO -= 1;
                    }                
  
                      if(preg_match("/^P-/",$sp1Array[0])){
                          $sp1Array[0] = preg_replace("/^P-/", "",$sp1Array[0]);
                          $numacylO -= 1;
                      }                
                      
                      if(preg_match("/^P-/",$sp2Array[0])){
                          $sp2Array[0] = preg_replace("/^P-/", "",$sp2Array[0]);
                          $numacylO -= 1;
                      }                
                                          
                      if(preg_match("/^P-/",$sp3Array[0])){
                          $sp3Array[0] = preg_replace("/^P-/", "",$sp3Array[0]);
                          $numacylO -= 1;
                    } 
                    
                    if($sp1Array[0] > 0){
                        $numacylC += $sp1Array[0];
                        $numacylO += 1;
                        $numacylH += (($sp1Array[0] - 1) * 2) - ($sp1Array[1] * 2);
                   }                  
                    if($sp2Array[0] > 0){
                        $numacylC += $sp2Array[0];
                        $numacylO += 1;
                        $numacylH += (($sp2Array[0] - 1) * 2) - ($sp2Array[1] * 2);
                   }                    
                    
                    if($sp3Array[0] > 0){
                        $numacylC += $sp3Array[0];
                        $numacylO += 1;
                        $numacylH += (($sp3Array[0] - 1) * 2) - ($sp3Array[1] * 2);
                   }  

##from glycerol backbone                   
                    $numC = $numacylC + 3;
                    $numH = $numacylH + 8;
                    $numO = $numacylO + 3;
                    $numDB = $sp1Array[1] + $sp2Array[1] + $sp3Array[1];

$GL_mass = $mult_factor * ($sub_exactmass + ($numC * 12) + ($numH * 1.007825) + ($numO * 15.994915));
$GL_mass = sprintf('%0.4f',$GL_mass);                   

include("ioncalc2.php");

$formula=preg_replace("/([A-Za-z])1$/","$1",$formula);
$formula=preg_replace("/([A-Za-z]+)1([A-Za-z]+)/","$1$2",$formula);
  ##########################################
  
  if(preg_match("/\/0:0\/0:0$/",$GL_abbrev)){
  $full_abbrev="MG(".$GL_abbrev.")";
  $GL_type="MG";
  }
  elseif(preg_match("/^0:0\/0:0/",$GL_abbrev)){
  $full_abbrev="MG(".$GL_abbrev.")";
  $GL_type="MG";
  }
  elseif(preg_match("/^0:0\/.+\/0:0$/",$GL_abbrev)){
  $full_abbrev="MG(".$GL_abbrev.")";
  $GL_type="MG";
  }  
  elseif(preg_match("/\/0:0$/",$GL_abbrev)){
  $full_abbrev="DG(".$GL_abbrev.")";
  $GL_type="DG";
  }
  elseif(preg_match("/^0:0/",$GL_abbrev)){
  $full_abbrev="DG(".$GL_abbrev.")";
  $GL_type="DG";
  }
  elseif(preg_match("/^0:0/",$GL_abbrev)  && preg_match("/\/0:0$/",$GL_abbrev)){
  $full_abbrev="MG(".$GL_abbrev.")";
  $GL_type="MG";
  }
  elseif(preg_match("/\/0:0\//",$GL_abbrev)){
  $full_abbrev="DG(".$GL_abbrev.")";
  $GL_type="DG";
  }
  else{
  $full_abbrev="TG(".$GL_abbrev.")";
  $GL_type="TG";
  }
     
  ############################################

     list($sn1val,$sn2val,$sn3val)=explode("\/",$GL_abbrev);
     list($sn1C,$sn1db)=explode(":",$sn1val);
     list($sn2C,$sn2db)=explode(":",$sn2val);
     list($sn3C,$sn3db)=explode(":",$sn3val);
     $sn1mass=($sn1C*12) + (($sn1C*2 -$sn1db*2)*1.007825) +(2*15.994915);
     $sn2mass=($sn2C*12) + (($sn2C*2 -$sn2db*2)*1.007825) +(2*15.994915);
     $sn3mass=($sn3C*12) + (($sn3C*2 -$sn3db*2)*1.007825) +(2*15.994915);


$ion=str_replace("+","plus",$theion);
$ion=str_replace("-","minus",$ion);
  $nominal_mass=floor($GL_mass);  
  if(preg_match("/([OP])-/",$full_abbrev,$MATCHES)){
  $CHAINTYPE=$MATCHES[1];
  $PREFIX=$CHAINTYPE .'-';
  }
  else{
  $CHAINTYPE='A';
  $PREFIX='';
  }
if($sn1 && $sn2 && $sn3){

if($sn1 == '0:0' && $sn2 == '0:0' && $sn3 == '0:0'){
#skip this one (no chains)
}
elseif(preg_match("/^[OP]-.+\/[OP]-/",$GL_abbrev)){
#ignore double ethers
}
elseif($chains == 'chains'){
print "$nominal_mass\t$GL_mass\t$numacylC\t$numDB\t$GL_type($GL_abbrev)\t$sn1\t$sn2\t$sn3\t$formula\t$GL_type\t$CHAINTYPE\t$theion\n";
}
else{
$full_abbrev_bulk="$GL_type($PREFIX$numacylC:$numDB)";
if(!in_array($full_abbrev_bulk,$uniq)){
array_push($uniq,$full_abbrev_bulk);
print "$nominal_mass\t$GL_mass\t$full_abbrev_bulk\t$numacylC\t$numDB\t$formula\t$GL_type\t$CHAINTYPE\t$theion\n";
}
}
}           
                   
                   
                    
		}
	}
}


?>

