<?php

# File: GP_expand.php Updated April 6th, 2017
# Author: Eoin Fahy
#
# Copyright (C) 2006-2014. The Regents of the University of California. All Rights Reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. Neither the name of the LIPID MAPS (TM) consortium nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The LIPID MAPS (TM) consortium is a multi-institutional multi-year effort
# involved in identification and characterization of existing and novel lipids,
# quantifications of changes in their metabolites, and development of biochemical
# pathways and interaction network maps involving lipids. Please acknowledge use
# of the LIPID MAPS (TM) Tools in your work by citing appropriate literature
# references. For further details, please visit www.lipidmaps.org.
#
# Support for this effort is provided by  by National Institutes of Health (NIH)
# and National Institute of General Medical Sciences (NIGMS) Glue Grant
# NIH/NIGMS Grant 1 U54 GM69338.
#
#USAGE: php GP_expand.php <none|chains> <neutral|ion type> <Minimum carbons in chain> <Maximum carbons in chain> <minimum ratio of carbons in chain to double bonds>
#USAGE: Permitted ion type : M.,M+H,M+2H,M+3H,M+4H,M-H,M-2H,M-3H,M-4H,M+NH4,M+Ag,M+Na,M+2Na,M+K,M+2K,M+Li,M+2Li,M.Cl,M.OAc,M-DFPU+NH4,M-diDFPU+NH4
#USAGE: ion types correspond to: [M.]+,[M+H]+,[M+2H]2+,[M+3H]3+,[M+4H]4+,[M-H]-,[M-2H]2-,[M-3H]3-,[M-4H]4-,[M+NH4]+,[M+Ag]+,[M+Na]+,[M+2Na]2+,[M+K]+,[M+2K]2+,[M+Li]+,[M+2Li]2+,[M+Cl]-,[M+OAc]-,[M-DFPU+NH4]+,[M-diDFPU+NH4]+
#Example: php GP_expand.php > GP.txt (neutral molecule mass calculation,individual chains are not specified)
#Example: php GP_expand.php chains > GP.txt (neutral molecule mass calculation,individual chains are specified)
#Example: php GP_expand.php none M+H > GP.txt ([M+H]+ ion mass calculation,individual chains are not specified)
#Example: php GP_expand.php none M-H 14 26 3.5 > GP.txt ([M-H]- ion mass calculation,individual chains are not specified, C14-C26 range for chains, minimum ratio of carbons:double bonds=3.5 )

$chains = isset($argv[1]) ? $argv[1] : '';
$ion = isset($argv[2]) ? $argv[2] : '';
$C_min = isset($argv[3]) ? $argv[3] : 10;
$C_max = isset($argv[4]) ? $argv[4] : 34;
$MINDBRATIO = isset($argv[5]) ? $argv[5] : 3.5;

$sub_exactmass= 0; 
$theion='';
$mult_factor=1;
$num_00=0;
$chaincount1=0;
$chaincount2=0;

include("ioncalc.php");
##############################SPECIFY CHAINS AND HEADGROUPS#################################

$sn1list='0:0
O-16:0
O-18:0
O-20:0
P-16:0
P-18:0
P-20:0
';

$hgArray=array('PA','PC','PS','PE','PI','PIP','PG');

##includes glycerol backbone  
    $headfile = array(
            'PA' => '3,9,0,6,1',
            'PC' => '8,20,1,6,1',
            'PS' => '6,14,1,8,1',
            'PE' => '5,14,1,6,1',
	    'PI' => '9,19,0,11,1',
            'PIP' => '9,20,0,14,2',
            'PG' => '6,15,0,8,1'
            );
         
##########################################################################################
for($i = $C_min; $i <= $C_max; $i++) {
for($j = 0; ($j/$i) <= 1/$MINDBRATIO; $j++) {
$sn1list .="$i" .':' . "$j\n";
}
}

$sn2list=preg_replace("/[OP]-[0-9\(\):EZ]+\n/","",$sn1list); #dont allow ethers at sn2
$GP2_list=explode("\n",$sn2list);

if(preg_match("/^0:0\n/",$sn2list) || preg_match("/\n0:0\n/",$sn2list)){
$num_00=1;
}

$GP1_list=explode("\n",$sn1list);
$num_chains =count($GP1_list) - $num_00;
file_put_contents('php://stderr', "Chain lengths from $C_min to $C_max. Number of chains: $num_chains\n");

$uniq=array();
$chaincount1=count($GP1_list);
$chaincount2=count($GP2_list);

for($i = 0; $i < count($hgArray); $i++) {
	file_put_contents('php://stderr', "$hgArray[$i]\n");
	for($j = 0; $j < $chaincount1-1; $j++) {
		for($k = 0; $k < $chaincount2-1; $k++) {
		
$HG_elements = explode(",",$headfile{$hgArray[$i]});


$full_abbrev="$hgArray[$i]". "(" . "$GP1_list[$j]" . "/" . "$GP2_list[$k]" .")";
$GP_abbrev="$GP1_list[$j]" . "/" . "$GP2_list[$k]";
$sn1=preg_replace("/\(.*/","",$GP1_list[$j]);
$sn2=preg_replace("/\(.*/","",$GP2_list[$k]);
$numC = 0;
$numH = 0;
$numO = 0;
$numN = 0;
$numP = 0;
$numNa = 0;
$numK = 0;
$numCl = 0;
$numAg = 0;
$numF = 0;
$numS = 0;
$numLi = 0;

$numacylC = 0;
$numacylH = 0;
$numacylO = 0;

$numDB = 0;

$sn1mass=0;
$sn2mass=0;

                    

$sp2Array = explode(":",$GP1_list[$j]);
$sp3Array = explode(":",$GP2_list[$k]);
                                  
                    if(preg_match("/^O-/",$sp2Array[0])){
                        $sp2Array[0] = preg_replace("/^O-/", "",$sp2Array[0]);
                        $numacylH += 2;
                        $numacylO -= 1;
                    }                
                                        
                    if(preg_match("/^O-/",$sp3Array[0])){
                        $sp3Array[0] = preg_replace("/^O-/", "",$sp3Array[0]);
                        $numacylH += 2;
                        $numacylO -= 1;
                    }                
                                        
               if(preg_match("/^P-/",$sp2Array[0])){
                          $sp2Array[0] = preg_replace("/^P-/", "",$sp2Array[0]);
                          $numacylO -= 1;
                      }                
                      
                      if(preg_match("/^P-/",$sp3Array[0])){
                          $sp3Array[0] = preg_replace("/^P-/", "",$sp3Array[0]);
                          $numacylO -= 1;
                      }   
                      
                    if($sp2Array[0] > 0){
                        $numacylC += $sp2Array[0];
                        $numacylO += 1;
                        $numacylH += (($sp2Array[0] - 1) * 2) - ($sp2Array[1] * 2);
                   }                    
                    
                    if($sp3Array[0] > 0){
                        $numacylC += $sp3Array[0];
                        $numacylO += 1;
                        $numacylH += (($sp3Array[0] - 1) * 2) - ($sp3Array[1] * 2);
                   }  
                   
                    $numC = $numacylC + $HG_elements[0];
                    $numH = $numacylH + $HG_elements[1];
                    $numN = $HG_elements[2];
                    $numO = $numacylO + $HG_elements[3];
                    $numP = $HG_elements[4];
                   $numDB = $sp2Array[1] + $sp3Array[1];

$GP_mass = $mult_factor * ($sub_exactmass + ($numC * 12) + ($numH * 1.007825) + ($numO * 15.994915) + ($numN * 14.003074) + ($numP * 30.973763));
$GP_mass = sprintf('%0.4f',$GP_mass);                   

include("ioncalc2.php");

$formula=preg_replace("/([A-Za-z])1$/","$1",$formula);
$formula=preg_replace("/([A-Za-z]+)1([A-Za-z]+)/","$1$2",$formula);
  

     list($sn1C,$sn1db)=explode(":",$GP1_list[$j]);
     list($sn2C,$sn2db)=explode(":",$GP2_list[$k]);
	$H20=18.010565;

     $sn1mass=($sn1C*12) + ((($sn1C*2 -$sn1db*2)-2)*1.007825) +(1*15.994915);
     $sn2mass=($sn2C*12) + ((($sn2C*2 -$sn2db*2)-2)*1.007825) +(1*15.994915);

$webion=str_replace("+","plus",$theion);
$webion=str_replace("-","minus",$webion);
$diacyl_abbrev=$GP1_list[$j] ."/" . $GP2_list[$k];

$nominal_mass=floor($GP_mass); 
  if(preg_match("/([OP])-/",$full_abbrev,$MATCHES)){
  $CHAINTYPE=$MATCHES[1];
  $PREFIX=$CHAINTYPE .'-';
  }
  else{
  $CHAINTYPE='A';
  $PREFIX='';
  }
if($sn1 && $sn2){

if($sn1 == '0:0' || $sn2 == '0:0'){
$thespecies= 'L' . $hgArray[$i];
}
else{
$thespecies= $hgArray[$i];
}
if($sn1 == '0:0' && $sn2 == '0:0'){
#skip this one (no chains)
}
elseif(preg_match("/^[OP]-.+\/[OP]-/",$GP_abbrev)){
#ignore double ethers
}
elseif($chains == 'chains'){
print "$nominal_mass\t$GP_mass\t$thespecies($GP_abbrev)\t$numacylC\t$numDB\t$sn1\t$sn2\t$formula\t$CHAINTYPE\t$theion\n";
}
else{
$full_abbrev_bulk="$thespecies($PREFIX$numacylC:$numDB)";
if(!in_array($full_abbrev_bulk,$uniq)){
array_push($uniq,$full_abbrev_bulk);
print "$nominal_mass\t$GP_mass\t$full_abbrev_bulk\t$numacylC\t$numDB\t$formula\t$CHAINTYPE\t$theion\n";
}
}
}
 


                   
                   
                    
		}
	}
}

?>

